/* pjmedia/include/pjmedia/config_auto.h.  Generated from config_auto.h.in by configure.  */
/*
 * Copyright (C) 2008-2009 Teluu Inc. (http://www.teluu.com)
 * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __PJMEDIA_CONFIG_AUTO_H_
#define __PJMEDIA_CONFIG_AUTO_H_

/**
 * @file config_auto.h
 * @brief PJMEDIA configuration as set by autoconf script
 */

/*
 * Note:
 *      The configuration in config_site.h overrides any other settings,
 *      including the setting as detected by autoconf. 
 */
 
/* G711 codec */
#ifndef PJMEDIA_HAS_G711_CODEC
/* #undef PJMEDIA_HAS_G711_CODEC */
#endif


#endif  /* __PJMEDIA_CONFIG_AUTO_H_ */



